@@ -10,12 +10,9 @@
 #include <cstdint>
 #include <string>
 
-VW_STD14_CONSTEXPR inline uint64_t hashall(const char* s, size_t len, uint64_t h)
-{
-  return VW::uniform_hash(s, len, h);
-}
+inline uint64_t hashall(const char* s, size_t len, uint64_t h) { return VW::uniform_hash(s, len, h); }
 
-VW_STD14_CONSTEXPR inline uint64_t hashstring(const char* s, size_t len, uint64_t h)
+inline uint64_t hashstring(const char* s, size_t len, uint64_t h)
 {
   const char* front = s;
   while (len > 0 && front[0] <= 0x20 && static_cast<int>(front[0]) >= 0)
